<?php
/**************************************************************************
 * $Id: StudIPortAccess.lib.php 4428 2010-11-15 10:14:28Z weyer $
 *
 * Copyright (C) 2008, 2009, 2010; Hamburg University of Technology
 *
 * > TECHNICAL REALIZATION
 *   Christian Renner <christian.renner@tu-harburg.de>
 *   Leo Krueger
 *   Sigurd Spieckermann
 *
 * > CONCEPT
 *   Peter Baasch
 *   Detlev Bieler
 *   Thomas Czerwionka
 *   Thomas Hapke
 *   Soenke Knutzen
 *   Corinna Peters
 *   Christian Renner
 *   Volker Turau
 *   Olav Wilde
 *
 **************************************************************************
 * This file is part of "StudIPort 2.0 ePortfolio".
 *
 * "StudIPort 2.0 ePortfolio" is free software: you can redistribute it
 * and/or modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation, either version 3 of the
 * License, or (at your option) any later version.
 *
 * "StudIPort 2.0 ePortfolio" is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with "StudIPort 2.0 ePortfolio".
 * If not, see <http://www.gnu.org/licenses/>.
 *
 *************************************************************************/


require_once("StudIPortTools.lib.php");
require_once('StudIPortDB.system.php');

/**
 * validate file access
 * 1.) check, if file_id provided
 * 2.) check, if file exists in DB
 * 3.) check, if file owner and current user match
 *
 * @param file_id identifier of the queried file
 * @param user_id identifier of current user
 * @param res JSON result array, used to write error messages
 *
 * @return true, if all checks succeed; false else
 */
function validateFileAccess($file_id, $user_id, &$res) {
	// check, if file_id provided at all
	$file_id = intval($file_id);
	
	if (empty($file_id)) {
		setErrorMsg($res, "Keine Datei gewählt");
		return false;
	}

	// obtain user_id belonging to provided file_id
	$db = new StudIPortDB();

	if ($user_id != StudIPortUser::NOBODY) {
		$db->queryf("SELECT user_id FROM studiport_file WHERE file_id='%u' AND user_id='%s'", $file_id, $user_id);
	} elseif (isset($_SESSION['StudIPortExtAcc']['access_code'])) {
		$db->queryf("SELECT    file.user_id " .
				   "FROM      studiport_file       AS file " .
				   "LEFT JOIN studiport_view_files AS viewFiles USING(file_id) " .
				   "LEFT JOIN studiport_view       AS view      USING(view_id) " .
				   "WHERE     file.file_id=%u AND view.access_code='%s'", $file_id, $_SESSION['StudIPortExtAcc']['access_code']);
	} else {
		setErrorMsg($res, "Sie sind weder eingeloggt noch haben Sie keinen Zugangscode für den externen Zugang eingegeben!");
		return false;
	}

	// does file exist/has user access rights?
	if ($db->num_rows() == 0) {
		setErrorMsg($res, "Datei existiert nicht oder Benutzer hat keine Berechtigung!");
		return false;
	}

	// all checks passed, so call is valid
	return true;
}


/**
 * validate view access
 * 1.) check, if view_id provided
 * 2.) check, if view exists in DB
 * 3a.) role=OWNER check, if view owner and current user match
 * 3b.) role=ALIEN check, if current user has access rights
 *
 * @param view_id identifier of the queried view
 * @param user_id identifier of current user
  @param res JSON result array, used to write error messages
 * @param role role of user (access privs)
 *
 * @return true, if all checks succeed; false else
 */
define("VIEW_ROLE_OWNER", 'owner');
define("VIEW_ROLE_ALIEN", 'alien');
function validateViewAccess($view_id, $user_id, &$res, $role = VIEW_ROLE_OWNER) {
	// check, if view_id provided at all
	$view_id = intval($view_id);
	if (empty($view_id)) {
		setErrorMsg($res, "Keine Ansicht gewählt");
		return false;
	}

	// prepare database connection
	$db = new StudIPortDB();

	// check, if user is owner
	$db->queryf("SELECT user_id FROM studiport_view WHERE view_id='%u'", $view_id);

	// does view exist?
	if ($db->num_rows() == 0) {
		setErrorMsg($res, "Ansicht existiert nicht");
		return false;
	}

	// now check that user is owner of the view
	// NOTE be paranoid here and assume OWNER generally
	$db->next_record();
	if (StudIPortUser::theUser()->getUserId() === $db->f('user_id')) {
		return true;
	} else if ($role === VIEW_ROLE_ALIEN) {
		$db->queryf("SELECT access.user_id"
			. " FROM studiport_view"
			. " LEFT JOIN studiport_view_access AS access USING (view_id)"
			. " WHERE view_id=%u"
			. "   AND access.user_id='%s'"
			. "   AND (NOT expires OR expires>=CURDATE())", $view_id, $user_id);
		$db->next_record();
		if ($db->num_rows() == 1 && StudIPortUser::theUser()->getUserId() === $db->f('user_id')) {
			return true;
		}
	}

	setErrorMsg($res, "Keine Berechtigung");
	return false;
}


/**
 * validate question access
 * 1.) check, if question_id provided
 * 2.) check, if question exists in DB
 * 3a.) role=OWNER check, if question owner and current user match
 * 3b.) role=ALIEN check, if current user has access rights
 *
 * @param question_id identifier of the queried question
 * @param user_id identifier of current user
 * @param res JSON result array, used to write error messages
 * @param role role of user (access privs)
 *
 * @return true, if all checks succeed; false else
 */
define("QUESTION_ROLE_OWNER", 'owner');
define("QUESTION_ROLE_ALIEN", 'alien');
function validateQuestionAccess($question_id, $user_id, &$res, $role = QUESTION_ROLE_OWNER) {
	// check, if question_id provided at all
	$question_id = intval($question_id);
	if (empty($question_id)) {
		setErrorMsg($res, "Keine Feedback-Frage gewählt");
		return false;
	}

	// prepare database connection
	$db = new StudIPortDB();

	// check, if user is owner
	$db->queryf("SELECT ".
	           "  user_id ".
	           "FROM studiport_feedback_question ".
	           "  LEFT JOIN studiport_view USING(view_id) ".
	           "WHERE question_id=%u", $question_id);

	// does question exist?
	if ($db->num_rows() == 0) {
		setErrorMsg($res, "Feedback-Frage existiert nicht");
		return false;
	}

	// now check that user is owner of the question
	// NOTE be paranoid here and assume OWNER generally
	$db->next_record();
	if (StudIPortUser::theUser()->getUserId() === $db->f('user_id')) {
		return true;
	} else if ($role === QUESTION_ROLE_ALIEN) {
		$db->queryf("(SELECT user_id"
			. " FROM studiport_feedback_request"
			. " WHERE question_id=%u)"
		        . " UNION"
		        . " (SELECT user_id"
			. " FROM studiport_feedback_response"
			. " WHERE question_id=%u)", $question_id, $question_id);
		$db->next_record();
		if ($db->num_rows() == 1 && StudIPortUser::theUser()->getUserId() === $db->f('user_id')) {
			return true;
		}
	}

	setErrorMsg($res, "Keine Berechtigung");
	return false;
}


/**
 * validate response access
 * 1.) check, if response_id provided
 * 2.) check, if response exists in DB
 * 3a.) role=SENDER
 * 3b.) role=RECEIVER
 * 3c.) role=ALIEN
 *
 * @param response_id identifier of the queried response
 * @param user_id identifier of current user
 * @param res JSON result array, used to write error messages
 * @param role role of user (access privs)
 *
 * @return true, if all checks succeed; false else
 */
define("RESPONSE_ROLE_SENDER",   0x01);
define("RESPONSE_ROLE_RECEIVER", 0x02);
define("RESPONSE_ROLE_ALIEN",    0x04);
function validateResponseAccess($response_id, $user_id, &$res, $role) {
	// check, if response_id provided at all
	$response_id = intval($response_id);
	if (empty($response_id)) {
		setErrorMsg($res, "Keine Antwort gewählt");
		return false;
	}

	// prepare database connection
	$db = new StudIPortDB();

	// check, if user is owner
	$db->queryf("SELECT ".
	           "  user_id ".
	           "FROM studiport_feedback_response ".
	           "WHERE response_id=%u", $response_id);

	// does response exist?
	if ($db->num_rows() == 0) {
		setErrorMsg($res, "Antwort existiert nicht");
		return false;
	}

	// now check if user is owner of the response (or has access)
	// NOTE be paranoid here and assume OWNER generally
	$db->next_record();
	if (($role & RESPONSE_ROLE_SENDER) && StudIPortUser::theUser()->getUserId() === $db->f('user_id')) {
		return true;
	} else if ($role & RESPONSE_ROLE_RECEIVER) {
		// TODO this only checks, if the current user has put the response
		// in fact, we'll need some more advanced checks for some use cases
		$db->queryf("SELECT view.user_id"
			. " FROM studiport_feedback_response"
		        . "   LEFT JOIN studiport_feedback_question USING(question_id)"
		        . "   LEFT JOIN studiport_view AS view USING(view_id)"
			. " WHERE response_id=%u", $response_id);
		$db->next_record();
		if ($db->num_rows() == 1 && StudIPortUser::theUser()->getUserId() === $db->f('user_id')) {
			return true;
		}
	}

	setErrorMsg($res, "Keine Berechtigung");
	return false;
}

?>
